// $Id: AtomDataModel.cpp,v 1.4 2014/04/25 09:12:20 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "AtomDataModel.h"
#include "AtomDataModel.moc"

#include "Debug.h"

//_______________________________________________
const QString AtomDataModel::columnTitles_[ AtomDataModel::nColumns ] =
{ tr( "Atom Name" ), tr( "Type" ), tr( "Count" ), tr( "Format" ), tr( "Length" ), tr( "Value" ) };

//__________________________________________________________________
QVariant AtomDataModel::data( const QModelIndex& index, int role ) const
{

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // retrieve associated color
    const AtomData::Pointer& atom( get( index ) );
    if( role == Qt::DisplayRole )
    {
        switch( index.column() )
        {
            case Name: return atom->name();
            case Type: return atom->type();
            case Count: return (int)atom->itemCount();
            case Format: return (int)atom->format();
            case Length: return (int)atom->length();
            case Value: return atom->stringValue();
            default: return QVariant();
        }

    } else if( role == Qt::TextAlignmentRole ){

        return (int)(Qt::AlignLeft|Qt::AlignTop);

    } else return QVariant();

}

//__________________________________________________________________
QVariant AtomDataModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < nColumns )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}

//____________________________________________________________
void AtomDataModel::_sort( int column, Qt::SortOrder order )
{
    Debug::Throw() << "ColorModel::sort - column: " << column << " order: " << order << endl;
    std::sort( _get().begin(), _get().end(), SortFTor( (ColumnType) column, order ) );
}

//________________________________________________________
bool AtomDataModel::SortFTor::operator () ( AtomData::Pointer first, AtomData::Pointer second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );

    switch( type_ )
    {

        case Name: return first->name() < second->name();
        case Type: return first->type() < second->type();
        case Count: return first->itemCount() < second->itemCount();
        case Format: return first->format() < second->format();
        case Length: return first->length() < second->length();
        case Value: return first->stringValue() < second->stringValue();
        default: return first < second;

    }

}
