// $Id: Menu.cpp,v 1.9 2014/05/06 08:33:47 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Menu.h"
#include "Menu.moc"

#include "Application.h"
#include "Debug.h"
#include "DebugMenu.h"
#include "DebugMenu.h"
#include "DefaultHelpText.h"
#include "DockWidget.h"
#include "File.h"
#include "HelpManager.h"
#include "HelpText.h"
#include "MainWindow.h"
#include "Singleton.h"
#include "WindowTreeWidget.h"
#include "XmlOptions.h"

#include <QMenu>

//__________________________________________________
Menu::Menu( QWidget* parent ):
QMenuBar( parent ),
Counter( "Menu" )
{

    Debug::Throw( "Menu::Menu" );

    // generic menu
    QMenu *menu;

    // application and top widget
    Application& application( *Singleton::get().application<Application>() );
    MainWindow& mainWindow( *static_cast<MainWindow*>(window()) );

    // file menu
    menu = addMenu( tr( "File" ) );
    menu->addAction( &mainWindow.clearAction() );
    menu->addAction( &mainWindow.reloadAction() );
    menu->addAction( &mainWindow.windowTreeWidget().reloadAction() );
    menu->addSeparator();
    menu->addAction( &mainWindow.saveIconAction() );
    menu->addSeparator();
    menu->addAction( &application.closeAction() );

    // Settings menu
    menu = addMenu( tr( "Settings" ) );
    menu->addAction( &mainWindow.windowTreeWidget().visibilityAction() );
    menu->addAction( &mainWindow.windowTreeWidget().showAllWindowsAction() );
    menu->addSeparator();
    menu->addAction( &application.configurationAction() );

    // help manager
    Base::HelpManager* help( new Base::HelpManager( this ) );
    help->setWindowTitle( tr( "Xgrabwindow Handbook" ) );
    help->install( helpText );
    help->install( Base::helpText, false );

    // create help menu
    menu = addMenu( tr( "Help" ) );
    menu->addAction( &help->displayAction() );
    menu->addSeparator();
    menu->addAction( &application.aboutAction() );
    menu->addAction( &application.aboutQtAction() );

    // debug menu
    menu->addSeparator();
    DebugMenu *debugMenu( new DebugMenu( menu ) );
    menu->addAction( debugMenu->menuAction() );
    debugMenu->menuAction()->setVisible( false );

}
