#ifndef WindowPropertiesDisplay_h
#define WindowPropertiesDisplay_h
// $Id: WindowPropertiesDisplay.h,v 1.6 2014/05/04 21:00:48 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "WindowProperties.h"
#include "AtomDataModel.h"
#include "IconSize.h"

#include <QFrame>
#include <QIcon>
#include <QLabel>
#include <QWidget>

class IconSizeMenu;
class GridLayoutItem;
class TreeView;

class WindowPropertiesDisplay: public QWidget, public Counter
{

    Q_OBJECT

    public:

    //! constructor
    WindowPropertiesDisplay( QWidget* );

    //! destructor
    virtual ~WindowPropertiesDisplay( void )
    {}

    //!@name accessors
    //@{

    //! icon
    QIcon icon( void ) const
    { return icon_; }

    //! class name
    QString windowClass( void ) const;

    //@}

    //!@name modifiers
    //@{

    //! clear
    void clear( void )
    { setWindowProperties( WindowProperties() ); }

    //! assign window properties
    void setWindowProperties( const WindowProperties& );

    //@}

    protected Q_SLOTS:

    //! update separator visibility
    void _updateSeparatorVisibility( void );

    //! raise icon size menu
    void _raiseIconSizeMenu( QPoint );

    //! update pixmap size
    void _updateIconSize( IconSize::Size );

    private Q_SLOTS:

    //! update configuration
    void _updateConfiguration( void );

    private:

    //! icon label
    QLabel* iconLabel_;

    //! name
    GridLayoutItem* nameItem_;

    //! class
    GridLayoutItem* classItem_;

    //! name
    GridLayoutItem* commandItem_;

    //! wid
    GridLayoutItem* widItem_;

    //! pid
    GridLayoutItem* pidItem_;

    //! size
    GridLayoutItem* sizeItem_;

    //! position
    GridLayoutItem* positionItem_;

    //! separator
    QFrame* separator_;

    //! current Icon
    QIcon icon_;

    //! icon sizes menu
    IconSizeMenu* iconSizeMenu_;

    //! icon size
    IconSize::Size iconSize_;

    //! atoms
    TreeView* list_;

    //! model
    AtomDataModel model_;

};

#endif
