#ifndef WindowSelectionFrame_h
#define WindowSelectionFrame_h

// $Id: WindowSelectionFrame.h,v 1.14 2014/05/01 16:46:29 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "Debug.h"
#include "WindowProperties.h"

#include <QBasicTimer>
#include <QCheckBox>
#include <QComboBox>
#include <QLabel>
#include <QMouseEvent>
#include <QPushButton>
#include <QSpinBox>
#include <QTimerEvent>

class LineEditor;
class XcbProperties;

//class WindowSelectionFrame:public QGroupBox, public Counter
class WindowSelectionFrame:public QWidget, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! creator
    WindowSelectionFrame( QWidget* parent );

    //! destructor
    ~WindowSelectionFrame( void )
    { Debug::Throw() << "WindowSelectionFrame::~WindowSelectionFrame.\n"; }

    //!@name accessors
    //@{

    //! window selection button
    QPushButton& selectWindowButton( void ) const
    { return *selectWindowButton_; }

    //! return selected window
    WId selectedWindow( void ) const
    { return window_; }

    //! window properties
    const WindowProperties& windowProperties( void ) const
    { return windowProperties_; }

    //@}

    //!@name modifiers
    //@{

    //! set delay
    void setDelay( int delay )
    { delaySpinBox_->setValue( delay ); }

    //@}

    //! event filter
    virtual bool eventFilter( QObject*, QEvent* );

    //! selection mode
    enum SelectionMode
    {
        SelectFromPointer,
        SelectRootWindow,
        SelectFromId
    };

    //! wid editor
    LineEditor& editor( void ) const
    { return *editor_; }

    Q_SIGNALS:

    //! emitted when new window is selected
    void windowSelectionChanged( void );

    public Q_SLOTS:

    //! select window
    void selectWindow( void );

    //! select root window
    void selectRootWindow( void );

    //! select window from WId editor
    void selectWindowFromEditor( void );

    //! select window from tree
    void selectWindowFromTree( WId );

    //! select window
    void selectWindowFromPointer( bool forceImmediatly = false );

    //! reload properties
    void reload( void );

    protected:

    //!@name event handlers
    //@{
    virtual void timerEvent( QTimerEvent* );
    //@}

    //! select window
    bool _selectWindowFromPointer( void );

    //! recursive find real window
    WId _findRealWindow( WId, int = 0 );

    //! clear capture
    void _clearCapture( void );

    private Q_SLOTS:

    //! selection mode
    void _selectionModeChanged( int );

    //! update configuration
    void _updateConfiguration( void );

    //! save configuration
    void _saveConfiguration( void );

    private:

    //! WID editor
    LineEditor* editor_;
    QLabel* editorLabel_;

    //! select window button
    QPushButton* selectWindowButton_;

    //! select root window
    QComboBox* selectionModeComboBox_;

    //! delay spin box
    QSpinBox* delaySpinBox_;
    QLabel* delayLabel_;

    //! auto hide checkbox
    QCheckBox* autoHideCheckBox_;
    QLabel* autoHideLabel_;

    //! delay timer
    QBasicTimer timer_;

    //! window grabber
    QDialog* captureWidget_;

    //! current selected color
    WId window_;

    //! current window properties
    WindowProperties windowProperties_;

    //! X11 properties
    XcbProperties* xcbProperties_;

    //! is true when the mouse is down
    bool mouseDown_;


};

#endif
