// $Id: WindowTreeModel.cpp,v 1.8 2014/05/06 08:10:03 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "WindowTreeModel.h"
#include "WindowTreeModel.moc"

#include "XcbUtil.h"

#include <QApplication>

//_______________________________________________________________
WindowTreeModel::WindowTreeModel( QObject* parent ):
    TreeModel<WindowTreeItem>( parent ),
    Counter( "WindowTreeModel" ),
    showAllWindows_( false )
{
    Debug::Throw( "WindowTreeModel::WindowTreeModel.\n" );
    setSortValues( false );
}

//__________________________________________________________________
QVariant WindowTreeModel::data( const QModelIndex& index, int role ) const
{

    if( !index.isValid() ) return QVariant();

    switch( role )
    {
        case Qt::DisplayRole:
        {

            const WindowTreeItem& item( _find( index.internalId() ).get() );
            switch( index.column() )
            {

                case Id: return XcbUtil::get().windowIdString( item.id() );
                case Parent: return XcbUtil::get().windowIdString( item.parentId() );
                case Name: return item.hasFlag( WindowTreeItem::Root ) ? tr( "The Root Window" ):item.windowName();
                default: return QVariant();

            }

            break;

        }

        case Qt::FontRole:
        {

            const WindowTreeItem& item( _find( index.internalId() ).get() );
            if( showAllWindows_ && ( item.flags() & (WindowTreeItem::Root|WindowTreeItem::Real ) ) )
            {
                QFont font( QApplication::font() );
                font.setBold( true );
                return font;
            } else return QVariant();
        }

        default: break;

    }

    return QVariant();

}


//__________________________________________________________________
QVariant WindowTreeModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    switch( role )
    {
        case Qt::DisplayRole:
        {
            switch( section )
            {
                case Id: return tr( "Window Id" );
                case Parent: return tr( "Parent Id" );
                case Name: return tr( "Window Name" );
                default: return QVariant();

            }
            break;
        }


        default: return QVariant();
    }

    // return empty
    return QVariant();

}

//___________________________________________________________
bool WindowTreeModel::SortFTor::operator() ( WindowTreeItem first, WindowTreeItem second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );
    switch( type_ )
    {
        case Id: return first.id() < second.id();
        case Parent:
        if( first.parentId() != second.parentId() ) return first.parentId() < second.parentId();
        else return first.id() < second.id();

        default: return first.id() < second.id();
    }

}
