#ifndef WindowTreeModel_h
#define WindowTreeModel_h

// $Id: WindowTreeModel.h,v 1.5 2014/05/06 08:10:03 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "Counter.h"
#include "WindowTreeItem.h"
#include "TreeModel.h"

//! WindowTreeItem model. Stores job information for display in lists
class WindowTreeModel : public TreeModel<WindowTreeItem>, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    WindowTreeModel(QObject *parent = 0);

    //! destructor
    virtual ~WindowTreeModel()
    {}

    //! filter real windows
    void setShowAllWindows( bool value )
    { showAllWindows_ = value; }

    //! column type enumeration
    enum ColumnType
    {
        Id,
        Parent,
        Name,
        nColumns
    };

    //!@name methods reimplemented from base class
    //@{

    // return job for a given index
    virtual QVariant data( const QModelIndex&, int ) const;

    //! header data
    virtual QVariant headerData( int, Qt::Orientation, int = Qt::DisplayRole ) const;

    //! number of columns for a given index
    virtual int columnCount(const QModelIndex& = QModelIndex()) const
    { return nColumns; }

    //@}

    protected:

    //! sorting
    virtual void _sort( int column, Qt::SortOrder order = Qt::AscendingOrder )
    { _root().sort( SortFTor( column, order ) ); }

    private:


    //! used to sort IconCaches
    class SortFTor: public ItemModel::SortFTor
    {

        public:

        //! constructor
        SortFTor( const int& type, Qt::SortOrder order = Qt::AscendingOrder ):
            ItemModel::SortFTor( type, order )
        {}

        //! prediction
        bool operator() ( const Item& first, const Item& second ) const
        { return (*this)( first.get(), second.get() ); }

        //! prediction
        bool operator() ( WindowTreeItem, WindowTreeItem ) const;

    };

    //! true if only real windows are to be shown
    bool showAllWindows_;

};

#endif
