// $Id: XcbProperties.cpp,v 1.5 2014/05/05 14:15:32 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "XcbProperties.h"

#include "AtomData.h"
#include "Debug.h"
#include "XcbUtil.h"

#if HAVE_X11

//_____________________________________________________________________
AtomData::Pointer XcbProperties::atomData( WId window, xcb_atom_t atom )
{

    AtomData::Pointer out( new AtomData() );

    // get atom name
    out->setName( atomName( atom ) );

    // xcb connection
    xcb_connection_t* connection( XcbUtil::get().connection<xcb_connection_t>() );

    // property
    const uint32_t maxLength = std::string().max_size();
    xcb_get_property_cookie_t cookie =
        xcb_get_property(
        connection, 0, window, atom,  XCB_ATOM_ANY, 0, (maxLength+3) / 4 );

    XcbUtil::ScopedPointer<xcb_get_property_reply_t> reply( xcb_get_property_reply( connection, cookie, 0x0 ) );
    out->setValid( reply && reply->type );
    if( !out->isValid() )
    {

        Debug::Throw() <<  "XcbProperties::atomData - window " << XcbUtil::get().windowIdString( window ) << " xcb_get_property failed." << endl;
        return out;

    }

    // format
    out->setFormat( reply.data()->format );

    // nbytes
    int nbytes(0);
    if( out->format() == 32 ) nbytes = sizeof( uint32_t );
    else if( out->format() == 16 ) nbytes = sizeof( uint16_t );
    else if( out->format() == 8 ) nbytes = sizeof( uint8_t );
    else if( out->format() == 0 ) nbytes = 0;

    // length, item count and type
    const int length( xcb_get_property_value_length( reply.data() ) );
    out->setLength( length );
    out->setItemCount( length/nbytes );
    out->setType( atomName( reply->type ) );

    // data
    out->setData( xcb_get_property_value( reply.data() ), length );

    return out;

}

//_____________________________________________________________________
QString XcbProperties::atomName( xcb_atom_t atom )
{
    // try find in map
    AtomNameMap::const_iterator iter( atomNames_.find( atom ) );
    if( iter != atomNames_.end() ) { return iter.value(); }

    // xcb connection
    xcb_connection_t* connection( XcbUtil::get().connection<xcb_connection_t>() );

    // print all supported atom names
    xcb_get_atom_name_cookie_t cookie( xcb_get_atom_name( connection, atom ) );
    XcbUtil::ScopedPointer<xcb_get_atom_name_reply_t> reply( xcb_get_atom_name_reply( connection, cookie, 0 ) );

    if( reply )
    {

        const QString name( QByteArray( xcb_get_atom_name_name( reply.data() ), xcb_get_atom_name_name_length( reply.data() ) ) );
        atomNames_.insert( atom, name );
        return name;

    } else return QString( tr( "Undefined atom %1" ) ).arg( XcbUtil::get().windowIdString( atom ) );

}

#endif
